"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const http = require("http-error-handler");
const defaultDescription = {
    populations: '',
    selection: '-__v',
    sortableBy: ['createdAt'],
    sortDefault: '-createdAt',
};
const validate = (validations = [], req) => {
    validations.forEach((path) => {
        const { name, type } = path;
        switch (type) {
            case 'Alpha':
                req.check(name, name + ' must be an alpha string').optional().isAlpha();
                break;
            case 'Alphanumeric':
                req.check(name, name + ' must be an alpha numeric string').optional().isAlphanumeric();
                break;
            case 'Boolean':
                req.check(name, name + ' must be a boolean').optional().isBoolean();
                break;
            case 'Email':
                req.check(name, name + ' must be an email address').optional().isEmail();
                break;
            case 'Empty':
                req.check(name, name + ' must not be set').isEmpty();
                break;
            case 'Float':
                req.check(name, name + ' must be an integer').optional().isFloat();
                break;
            case 'Int':
                req.check(name, name + ' must be an integer').optional().isInt();
                break;
            case 'Number':
                req.check(name, name + ' must be a number').optional().isDecimal();
                break;
        }
    });
};
const deleteEmptyBodyParams = (validations = [], body) => {
    validations.forEach((path) => {
        if (path.type === 'Empty') {
            delete body[path.name];
        }
    });
};
exports.create = (Resource, description = defaultDescription) => {
    return (req, res, next) => {
        Resource.schema.requiredPaths().forEach((path) => {
            req.checkBody(path, 'Parameter missing').notEmpty();
        });
        Resource.schema.eachPath((path, type) => {
            switch (type.instance) {
                case 'Boolean':
                    req.checkBody(path, path + ' must be a boolean').optional().isBoolean();
                    break;
                case 'Number':
                    req.checkBody(path, path + ' must be a number').optional().isDecimal();
                    break;
            }
        });
        validate(description.validations, req);
        deleteEmptyBodyParams(description.validations, req.body);
        req.getValidationResult()
            .then((result) => {
            result.throw();
            const resource = new Resource(req.body);
            return resource.save();
        })
            .then((resource) => res.status(201).json(resource))
            .catch(next);
    };
};
exports.read = (Resource, description = defaultDescription) => {
    return (req, res, next) => {
        if (description.identifier) {
            req.checkParams('id', 'Parameter missing').notEmpty();
        }
        else {
            req.checkParams('id', 'Parameter missing or invalid. Must be ObjectId').notEmpty().isMongoId();
        }
        validate(description.validations, req);
        const populations = req.query.populate === 'true' ? description.populations ? description.populations : '' : '';
        const identifier = description.identifier || '_id';
        req.getValidationResult()
            .then((result) => {
            result.throw();
            return Resource
                .findOne({ [identifier]: req.params.id })
                .populate(populations)
                .select(description.selection || '')
                .catch((err) => { throw http.internalError; });
        })
            .then((resource) => {
            if (resource) {
                return res.status(200).json(resource);
            }
            else {
                const notFound = Object.assign({}, http.notFound, { message: Resource.modelName + ' could not be found' });
                return res.status(notFound.code).json(notFound);
            }
        })
            .catch(next);
    };
};
exports.readAll = (Resource, description = defaultDescription) => {
    return (req, res, next) => {
        if (req.query.sort && (description.sortableBy || []).indexOf(req.query.sort) === -1) {
            const badRequest = Object.assign({}, http.badRequest, { message: 'Can not sort by ' + req.query.sort });
            return http.errorHandler(badRequest, req, res);
        }
        const sortField = req.query.sort ? req.query.sort : description.sortDefault;
        const sortOrder = req.query.order === 'DESC' ? '-' : '';
        const skip = req.query.skip ? +req.query.skip : 0;
        const limit = req.query.limit ? +req.query.limit : Number.MAX_SAFE_INTEGER;
        const all = req.query.all === 'true' ? {} : description.defaultFilter ? description.defaultFilter : {};
        const populations = req.query.populate === 'true' ? description.populations ? description.populations : '' : '';
        Resource
            .find(all)
            .skip(skip)
            .limit(limit)
            .sort(sortOrder + sortField)
            .populate(populations)
            .select(description.selection || '')
            .then((resources) => {
            res.set('X-Total-Count', String(resources.length));
            return res.status(200).json(resources);
        })
            .catch(next);
    };
};
exports.update = (Resource, description = defaultDescription) => {
    return (req, res, next) => {
        if (description.identifier) {
            req.checkParams('id', 'Parameter missing').notEmpty();
        }
        else {
            req.checkParams('id', 'Parameter missing or invalid. Must be ObjectId').notEmpty().isMongoId();
        }
        validate(description.validations, req);
        const identifier = description.identifier || '_id';
        req.getValidationResult()
            .then((result) => {
            result.throw();
            return Resource
                .findOne({ [identifier]: req.params.id })
                .catch((err) => { throw http.internalError; });
        })
            .then((resource) => {
            if (resource) {
                return Resource.update({ [identifier]: resource._id }, { $set: req.body });
            }
            else {
                throw Object.assign({}, http.notFound, { message: Resource.modelName + ' could not be found' });
            }
        })
            .then(() => res.status(200).end())
            .catch(next);
    };
};
exports.destroy = (Resource, description = defaultDescription) => {
    return (req, res, next) => {
        if (description.identifier) {
            req.checkParams('id', 'Parameter missing').notEmpty();
        }
        else {
            req.checkParams('id', 'Parameter missing or invalid. Must be ObjectId').notEmpty().isMongoId();
        }
        validate(description.validations, req);
        const identifier = description.identifier || '_id';
        req.getValidationResult()
            .then((result) => {
            result.throw();
            return Resource
                .remove({ [identifier]: req.params.id })
                .catch((err) => { throw http.internalError; });
        })
            .then(() => {
            return res.status(200).end();
        })
            .catch(next);
    };
};
